@@ -5,11 +5,19 @@ from __future__ import print_function
 from __future__ import division
 from __future__ import absolute_import
 
+from typing import Optional, Union
+
+try:
+    from typing import Literal
+except ImportError:
+    from typing_extensions import Literal
+
 import warnings
 import datetime
 
 import numpy as np
 import pandas as pd
+import typic
 
 from tstoolbox import tsutils
 from tstoolbox import tstoolbox
@@ -26,7 +34,9 @@ from .melodist.melodist.precipitation import disagg_prec
 warnings.filterwarnings("ignore")
 
 
-def single_target_units(source_units, target_units, default=None):
+@tsutils.transform_args(source_units=tsutils.make_list, target_units=tsutils.make_list)
+@typic.al
+def single_target_units(source_units, target_units, default=None, cnt=1):
     if default is None:
         return source_units
 
@@ -34,7 +44,7 @@ def single_target_units(source_units, target_units, default=None):
         return [default] * len(source_units)
 
     tunits = set(target_units)
-    if len(tunits) != 1:
+    if len(tunits) != cnt:
         raise ValueError(
             tsutils.error_wrapper(
                 """
@@ -45,39 +55,30 @@ a single "target_units".  You gave "{target_units}".
                 )
             )
         )
-    return target_units[0] * len(source_units)
+    if len(source_units) == len(target_units):
+        return target_units
+
+    return [target_units[0]] * len(source_units)
 
 
-@tsutils.validator(
-    method=[
-        str,
-        [
-            "domain",
-            [
-                "sine_min_max",
-                "sine_mean",
-                "sine",
-                "mean_course_min_max",
-                "mean_course_mean",
-            ],
-        ],
-        1,
-    ],
-    min_max_time=[str, ["domain", ["fix", "sun_loc", "sun_loc_shift"]], 1],
-    mod_nighttime=[bool, ["pass", []], 1],
-    lat=[float, ["range", [-90, 90]], 1],
-    lon=[float, ["range", [-180, 180]], 1],
-    temp_min_col=[[int, ["domain", [1, None]], 1], [str, ["pass", []], 1]],
-    temp_max_col=[[int, ["domain", [1, None]], 1], [str, ["pass", []], 1]],
-    temp_mean_col=[[int, ["domain", [1, None]], 1], [str, ["pass", []], 1]],
-    max_delta=[bool, ["domain", [True, False]], 1],
-    hourly=[str, ["pass", []], 1],
-)
+@typic.constrained(ge=-90, le=90)
+class FloatLatitude(float):
+    """ -90 <= float <= 90 """
+
+
+@typic.constrained(ge=-180, le=180)
+class FloatLongitude(float):
+    """ -180 <= float <= 180 """
+
+
+@typic.al
 def temperature(
-    method,
+    method: Literal[
+        "sine_min_max", "sine_mean", "sine", "mean_course_min_max", "mean_course_mean"
+    ],
     source_units,
-    min_max_time="fix",
-    mod_nighttime=False,
+    min_max_time: Literal["fix", "sun_loc", "sun_loc_shift"] = "fix",
+    mod_nighttime: bool = False,
     input_ts="-",
     start_date=None,
     end_date=None,
@@ -89,13 +90,13 @@ def temperature(
     names=None,
     print_input=False,
     target_units=None,
-    max_delta=False,
-    temp_min_col=None,
-    temp_max_col=None,
-    temp_mean_col=None,
-    lat=None,
-    lon=None,
-    hourly=None,
+    max_delta: bool = False,
+    temp_min_col: Optional[Union[tsutils.IntGreaterEqualToOne, str]] = None,
+    temp_max_col: Optional[Union[tsutils.IntGreaterEqualToOne, str]] = None,
+    temp_mean_col: Optional[Union[tsutils.IntGreaterEqualToOne, str]] = None,
+    lat: Optional[FloatLatitude] = None,
+    lon: Optional[FloatLongitude] = None,
+    hourly: Optional[str] = None,
 ):
     """Disaggregate daily to hourly data.
 
@@ -136,9 +137,6 @@ def temperature(
 
     max_delta = get_shift_by_data(temp_hourly, lon, lat, round(lon/15.0))
     """
-    source_units = tsutils.make_list(source_units)
-    target_units = tsutils.make_list(target_units)
-
     target_units = single_target_units(source_units, target_units, "degC")
 
     pd.options.display.width = 60
@@ -217,7 +215,7 @@ Instead `temp_min_col` is {temp_min_col} and `temp_max_col` is
     else:
         tsd.columns = ["tmin", "tmax"]
 
-    if any(tsd.tmax <= tsd.tmin):
+    if any((tsd.tmax <= tsd.tmin).dropna()):
         raise ValueError(
             tsutils.error_wrapper(
                 """
@@ -242,9 +240,9 @@ estimated by the average of `temp_min_col` and `temp_max_col`""".format(
                 )
             )
         )
-        tsd.temp = (tsd.tmin + tsd.tmax) / 2.0
+        tsd["temp"] = (tsd.tmin + tsd.tmax) / 2.0
 
-        if any(tsd.tmin >= tsd.temp) or any(tsd.tmax <= tsd.temp):
+        if any((tsd.tmin >= tsd.temp).dropna()) or any((tsd.tmax <= tsd.temp).dropna()):
             raise ValueError(
                 tsutils.error_wrapper(
                     """
@@ -308,25 +306,16 @@ You gave:
     return tsutils.return_input(print_input, tsd, ntsd)
 
 
-@tsutils.validator(
-    method=[
-        str,
-        [
-            "domain",
-            [
-                "equal",
-                "minimal",
-                "dewpoint_regression",
-                "linear_dewpoint_variation",
-                "min_max",
-                "month_hour_precip_mean",
-            ],
-        ],
-        1,
-    ],
-)
+@typic.al
 def humidity(
-    method,
+    method: Literal[
+        "equal",
+        "minimal",
+        "dewpoint_regression",
+        "linear_dewpoint_variation",
+        "min_max",
+        "month_hour_precip_mean",
+    ],
     source_units,
     input_ts="-",
     columns=None,
@@ -391,9 +380,6 @@ def humidity(
                   "min_max"]
         need HOURLY temperature in 'temp'
     """
-    source_units = tsutils.make_list(source_units)
-    target_units = tsutils.make_list(target_units)
-
     target_units = single_target_units(source_units, target_units, "")
 
     if method == "equal" and hum_mean_col is None:
@@ -547,9 +533,9 @@ required identified by the filename in keyword `hourly_temp`."""
     return tsutils.return_input(print_input, tsd, ntsd)
 
 
-@tsutils.validator(method=[str, ["domain", ["equal", "cosine", "random",],], 1,],)
+@typic.al
 def wind_speed(
-    method,
+    method: Literal["equal", "cosine", "random"],
     source_units,
     input_ts="-",
     columns=None,
@@ -581,9 +567,6 @@ def wind_speed(
         b: parameter b for the cosine function
         t_shift: parameter t_shift for the cosine function
     """
-    source_units = tsutils.make_list(source_units)
-    target_units = tsutils.make_list(target_units)
-
     target_units = single_target_units(source_units, target_units, "m/s")
 
     target_units = target_units[0] * len(source_units)
@@ -628,15 +611,9 @@ t_shift = {t_shift}
     )
 
 
-@tsutils.validator(
-    method=[
-        str,
-        ["domain", ["pot_rad", "pot_rad_via_ssd", "pot_rad_via_bc", "mean_course",],],
-        1,
-    ],
-)
+@typic.al
 def radiation(
-    method,
+    method: Literal["pot_rad", "pot_rad_via_ssd", "pot_rad_via_bc", "mean_course"],
     source_units,
     input_ts="-",
     columns=None,
@@ -695,9 +672,6 @@ def radiation(
         bristcamp_a
         bristcamp_c
     """
-    source_units = tsutils.make_list(source_units)
-    target_units = tsutils.make_list(target_units)
-
     target_units = single_target_units(source_units, target_units, "W/m2")
 
     target_units = target_units[0] * len(source_units)
@@ -776,14 +750,11 @@ and `bristcamp_c`."""
     )
 
 
-@tsutils.validator(
-    method=[str, ["domain", ["equal", "cascade", "masterstation",],], 1,],
-)
+@typic.al
 def precipitation(
-    method,
+    method: Literal["equal", "cascade", "masterstation"],
     source_units,
     input_ts="-",
-    columns=None,
     start_date=None,
     end_date=None,
     dropna="no",
@@ -794,11 +765,10 @@ def precipitation(
     names=None,
     target_units=None,
     print_input=False,
+    columns=None,
+    masterstation_hour_col: Optional[Union[tsutils.IntGreaterEqualToOne, str]] = None,
 ):
     """Disaggregate daily to hourly data."""
-    source_units = tsutils.make_list(source_units)
-    target_units = tsutils.make_list(target_units)
-
     target_units = single_target_units(source_units, target_units, "mm")
 
     pd.options.display.width = 60
@@ -817,17 +787,49 @@ def precipitation(
         clean=clean,
     )
 
-    return tsutils.return_input(
-        print_input, tsd, pd.DataFrame(disagg_prec(tsd, method=method,))
-    )
+    if method == "masterstation":
+        try:
+            # If masterstations_hour_col is a column name:
+            masterstation_hour_col = tsd.columns.get_loc(masterstation_hour_col)
+        except KeyError:
+            # If masterstations_hour_col is a column number:
+            masterstation_hour_col = int(masterstation_hour_col) - 1
+
+        masterstation_hour_col = tsd.columns[masterstation_hour_col]
+
+        # Should only be one hourly column in the input.
+        mhour = tsd[masterstation_hour_col].to_frame()
+        dsum = mhour.groupby(pd.Grouper(freq="D")).sum().asfreq("H", method="ffill")
+        master = mhour.join(dsum, rsuffix="sum")
+        mask = master.iloc[:, 0] > 0.0
+        master = (
+            master.loc[mask, master.columns[0]] / master.loc[mask, master.columns[1]]
+        ).to_frame()
+        print(master)
+        ntsd = tsd.loc[:, tsd.columns != masterstation_hour_col].asfreq(
+            "H", method="ffill"
+        )
+        print(ntsd)
+        ntsd = ntsd.join(master)
+        print(ntsd)
+        ntsd = ntsd.loc[:, tsd.columns != masterstation_hour_col].multiply(
+            ntsd.iloc[:, -1:], axis="index"
+        )
+        print(ntsd)
+        sys.exit()
+        # All the remaining columns are daily.
+        ntsd = (
+            tsd.loc[:, tsd.columns != masterstation_hour_col]
+            .asfreq("H", method="ffill")
+            .mul(master, axis="rows")
+        )
+
+    return tsutils.return_input(print_input, tsd, ntsd)
 
 
-@tsutils.validator(
-    method=[str, ["domain", ["trap", "fixed"]], 1],
-    lat=[float, ["range", [-90, 90]], 1],
-)
+@typic.al
 def evaporation(
-    method,
+    method: Literal["trap", "fixed"],
     source_units,
     input_ts="-",
     columns=None,
@@ -841,12 +843,9 @@ def evaporation(
     names=None,
     target_units=None,
     print_input=False,
-    lat=None,
+    lat: Optional[FloatLatitude] = None,
 ):
     """Disaggregate daily to hourly data."""
-    source_units = tsutils.make_list(source_units)
-    target_units = tsutils.make_list(target_units)
-
     target_units = single_target_units(source_units, target_units)
 
     pd.options.display.width = 60
